import UIKit

import PlaygroundSupport
PlaygroundPage.current.needsIndefiniteExecution = true

let userQ = DispatchQueue.global(qos: .userInitiated)
let mainQ = DispatchQueue.main

func fetchData() {
  print(">> Fetch data start")
  sleep(2)
  print(">> Fetch data end")
}

func updateUI() {
  print(">> UI Update start")
  print("Updating UI")
  print(">> UI Update end")
}


// Using global dipatch queue
userQ.async {
  fetchData()
  mainQ.async {
    updateUI()
  }
}


// Using Dispatch Work Items
let fetchDataWorkItem = DispatchWorkItem {
  fetchData()
}

let updateUIWorkItem = DispatchWorkItem {
  updateUI()
}

userQ.async(execute: fetchDataWorkItem)
fetchDataWorkItem.notify(queue: .main, execute: updateUIWorkItem)


sleep(5)
PlaygroundPage.current.finishExecution()
